"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var path_1 = tslib_1.__importDefault(require("path"));
var ora_1 = tslib_1.__importDefault(require("ora"));
var fs_extra_1 = tslib_1.__importDefault(require("fs-extra"));
var enquirer_1 = require("enquirer");
var add_1 = require("../../setup/add");
var package_const_1 = require("../../shared/constants/package.const");
var messages_1 = require("../../shared/messages");
var utils_1 = require("../../shared/utils");
var featureOptions = Object.keys(add_1.featureSetup);
exports.addBinCommand = function (prog) {
    prog
        .command('add [featureName]')
        .describe("Add available feature.\n  Currently available choices: [" + featureOptions.join(', ') + "]")
        .example('add')
        .example('add playground')
        .action(function (featureName) {
        if (featureName === void 0) { featureName = ''; }
        return tslib_1.__awaiter(void 0, void 0, void 0, function () {
            var _a, failed, successful, adding, exists, invalidFeatureName, featureOption, featureNamePrompt, packageDir, packageJsonPath, packageJson, bootSpinner, err_1;
            return tslib_1.__generator(this, function (_b) {
                switch (_b.label) {
                    case 0:
                        _a = new messages_1.FeatureMessages(), failed = _a.failed, successful = _a.successful, adding = _a.adding, exists = _a.exists, invalidFeatureName = _a.invalidFeatureName;
                        featureOption = featureName;
                        if (!!featureOptions.includes(featureOption)) return [3 /*break*/, 2];
                        console.log(invalidFeatureName(featureOption));
                        featureNamePrompt = new enquirer_1.Select({
                            message: 'Choose a feature',
                            choices: featureOptions.map(function (option) { return ({
                                name: option,
                                message: utils_1.info(option)
                            }); })
                        });
                        return [4 /*yield*/, featureNamePrompt.run()];
                    case 1:
                        featureOption = (_b.sent());
                        _b.label = 2;
                    case 2: return [4 /*yield*/, utils_1.findWorkspacePackageDir()];
                    case 3:
                        packageDir = _b.sent();
                        packageJsonPath = path_1.default.resolve(packageDir, package_const_1.PACKAGE_JSON);
                        return [4 /*yield*/, fs_extra_1.default.readJSON(packageJsonPath)];
                    case 4:
                        packageJson = (_b.sent());
                        bootSpinner = ora_1.default(adding(featureOption));
                        bootSpinner.start();
                        _b.label = 5;
                    case 5:
                        _b.trys.push([5, 8, , 9]);
                        return [4 /*yield*/, fs_extra_1.default.copy(path_1.default.resolve(__dirname, "../../../../templates/add/" + featureOption), path_1.default.resolve(packageDir, add_1.featureSetup[featureOption].path), { overwrite: false, errorOnExist: true })];
                    case 6:
                        _b.sent();
                        return [4 /*yield*/, fs_extra_1.default.outputJSON(packageJsonPath, tslib_1.__assign(tslib_1.__assign({}, packageJson), { scripts: tslib_1.__assign(tslib_1.__assign({}, packageJson.scripts), add_1.featureSetup[featureOption].scripts) }), { spaces: 2 })];
                    case 7:
                        _b.sent();
                        bootSpinner.succeed(successful(featureOption));
                        return [3 /*break*/, 9];
                    case 8:
                        err_1 = _b.sent();
                        bootSpinner.fail(failed(featureOption));
                        if (err_1.toString().includes('already exists')) {
                            console.log(exists());
                        }
                        utils_1.logError(err_1);
                        process.exit(1);
                        return [3 /*break*/, 9];
                    case 9: return [2 /*return*/];
                }
            });
        });
    });
};
