"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var path_1 = tslib_1.__importDefault(require("path"));
var ora_1 = tslib_1.__importDefault(require("ora"));
var fs_extra_1 = tslib_1.__importDefault(require("fs-extra"));
var enquirer_1 = require("enquirer");
var utils_1 = require("../../shared/utils");
var setup_1 = require("../../setup");
var messages_1 = require("../../shared/messages");
var package_const_1 = require("../../shared/constants/package.const");
var templateOptions = Object.keys(setup_1.generateSetup);
var featureOptions = Object.keys(setup_1.featureSetup);
exports.generateBinCommand = function (prog) {
    prog
        .command('generate <pkg>')
        .describe('Generate a new package.')
        // eslint-disable-next-line @typescript-eslint/ban-ts-ignore
        // @ts-ignore
        .alias('g')
        .example('generate packageName')
        .option('t, template', "Specify a template.\n     Available templates: [" + templateOptions.join(', ') + "]\n     \n     ")
        .example("generate packageName --template " + templateOptions[0])
        .option('f, feature', "Specify a feature.\n     Available features: [" + featureOptions.join(', ') + "]\n     \n     ")
        .example("generate packageName --feature " + featureOptions[0])
        .action(function (pkgName, _a) {
        var template = _a.template;
        return tslib_1.__awaiter(void 0, void 0, void 0, function () {
            function getPackagePath(projectPath) {
                return tslib_1.__awaiter(this, void 0, void 0, function () {
                    var isExist, packageNamePrompt, nextProjectPath;
                    return tslib_1.__generator(this, function (_a) {
                        switch (_a.label) {
                            case 0: return [4 /*yield*/, fs_extra_1.default.pathExists(projectPath)];
                            case 1:
                                isExist = _a.sent();
                                if (!isExist) {
                                    return [2 /*return*/, projectPath];
                                }
                                bootSpinner.fail(failed());
                                packageNamePrompt = new enquirer_1.Input({
                                    message: exists(),
                                    initial: copy(),
                                    result: function (v) { return v.trim(); }
                                });
                                return [4 /*yield*/, packageNamePrompt.run()];
                            case 2:
                                packageName = _a.sent();
                                changePackageName(packageName);
                                nextProjectPath = workspaceRoot + "/" + packageSetupPath + "/" + packageName;
                                return [2 /*return*/, getPackagePath(nextProjectPath)];
                        }
                    });
                });
            }
            var packageName, _b, changePackageName, successfulConfigure, failedConfigure, successful, copy, failed, generating, exists, invalidTemplate, preparedPackage, preparingPackage, packageTemplateType, workspaceRoot, packageJsonPath, _c, rootName, workspaces, license, workspacePackages, packageSetupPath, bootSpinner, projectPath, prompt, author, licenseInput, templateConfig, generatePackageJson, pkgJson, err_1, dependencies, preparingSpinner, _d, _e, err_2;
            return tslib_1.__generator(this, function (_f) {
                switch (_f.label) {
                    case 0:
                        packageName = pkgName;
                        _b = new messages_1.GenerateMessages(packageName), changePackageName = _b.changePackageName, successfulConfigure = _b.successfulConfigure, failedConfigure = _b.failedConfigure, successful = _b.successful, copy = _b.copy, failed = _b.failed, generating = _b.generating, exists = _b.exists, invalidTemplate = _b.invalidTemplate, preparedPackage = _b.preparedPackage, preparingPackage = _b.preparingPackage;
                        return [4 /*yield*/, utils_1.findWorkspaceRootDir()];
                    case 1:
                        workspaceRoot = _f.sent();
                        packageJsonPath = path_1.default.resolve(workspaceRoot, package_const_1.PACKAGE_JSON);
                        return [4 /*yield*/, fs_extra_1.default.readJSON(packageJsonPath)];
                    case 2:
                        _c = (_f.sent()), rootName = _c.name, workspaces = _c.workspaces, license = _c.license;
                        workspacePackages = utils_1.getWorkspacePackagePaths(workspaces);
                        packageSetupPath = utils_1.getWorkspacePackageSetupPath(workspacePackages);
                        bootSpinner = ora_1.default(generating());
                        _f.label = 3;
                    case 3:
                        _f.trys.push([3, 14, , 15]);
                        return [4 /*yield*/, getPackagePath(workspaceRoot + "/" + packageSetupPath + "/" + packageName)];
                    case 4:
                        projectPath = _f.sent();
                        prompt = new enquirer_1.Select({
                            message: 'Choose a template',
                            choices: templateOptions.map(function (option) { return ({
                                name: option,
                                message: utils_1.info(option)
                            }); })
                        });
                        if (!template) return [3 /*break*/, 7];
                        packageTemplateType = template.trim();
                        if (!!prompt.choices.find(function (choice) { return choice.name === packageTemplateType; })) return [3 /*break*/, 6];
                        bootSpinner.fail(invalidTemplate(packageTemplateType));
                        return [4 /*yield*/, prompt.run()];
                    case 5:
                        packageTemplateType = _f.sent();
                        _f.label = 6;
                    case 6: return [3 /*break*/, 9];
                    case 7: return [4 /*yield*/, prompt.run()];
                    case 8:
                        packageTemplateType = _f.sent();
                        _f.label = 9;
                    case 9:
                        bootSpinner.start();
                        return [4 /*yield*/, fs_extra_1.default.copy(path_1.default.resolve(__dirname, "../../../../templates/generate/" + packageTemplateType), projectPath, {
                                overwrite: true
                            })];
                    case 10:
                        _f.sent();
                        author = utils_1.getAuthorName();
                        if (!!author) return [3 /*break*/, 12];
                        bootSpinner.stop();
                        licenseInput = new enquirer_1.Input({
                            name: 'author',
                            message: 'Who is the package author?'
                        });
                        return [4 /*yield*/, licenseInput.run()];
                    case 11:
                        author = _f.sent();
                        utils_1.setAuthorName(author);
                        bootSpinner.start();
                        _f.label = 12;
                    case 12:
                        process.chdir(projectPath);
                        templateConfig = setup_1.generateSetup[packageTemplateType];
                        generatePackageJson = setup_1.composePackageJson(templateConfig);
                        pkgJson = generatePackageJson({
                            author: author,
                            name: packageName,
                            rootName: rootName,
                            license: license
                        });
                        return [4 /*yield*/, fs_extra_1.default.outputJSON(path_1.default.resolve(projectPath, package_const_1.PACKAGE_JSON), pkgJson, {
                                spaces: 2
                            })];
                    case 13:
                        _f.sent();
                        bootSpinner.succeed(successful());
                        return [3 /*break*/, 15];
                    case 14:
                        err_1 = _f.sent();
                        bootSpinner.fail(failed());
                        utils_1.logError(err_1);
                        process.exit(1);
                        return [3 /*break*/, 15];
                    case 15:
                        dependencies = setup_1.generateSetup[packageTemplateType].dependencies;
                        preparingSpinner = ora_1.default(preparingPackage(dependencies.sort())).start();
                        _f.label = 16;
                    case 16:
                        _f.trys.push([16, 20, , 21]);
                        return [4 /*yield*/, utils_1.sortPackageJson()];
                    case 17:
                        _f.sent();
                        return [4 /*yield*/, utils_1.buildPackage()];
                    case 18:
                        _f.sent();
                        preparingSpinner.succeed(successfulConfigure());
                        _e = (_d = console).log;
                        return [4 /*yield*/, preparedPackage(packageName)];
                    case 19:
                        _e.apply(_d, [_f.sent()]);
                        return [3 /*break*/, 21];
                    case 20:
                        err_2 = _f.sent();
                        preparingSpinner.fail(failedConfigure());
                        utils_1.logError(err_2);
                        process.exit(1);
                        return [3 /*break*/, 21];
                    case 21: return [2 /*return*/];
                }
            });
        });
    });
};
