"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var execa_1 = tslib_1.__importDefault(require("execa"));
var ora_1 = tslib_1.__importDefault(require("ora"));
var messages_1 = require("../../shared/messages");
var utils_1 = require("../../shared/utils");
exports.installBinCommand = function (prog) {
    prog
        .command('install')
        .describe('Install one or more dependencies to the workspace root. Run this script inside any package and re-space will add peer dependencies as well.')
        // eslint-disable-next-line @typescript-eslint/ban-ts-ignore
        // @ts-ignore
        .alias('i')
        .example('install libraryName')
        .option('D, dev', 'Install development dependencies')
        .example("install libraryName --dev")
        .action(function (_a) {
        var dependencies = _a._, dev = _a.dev;
        return tslib_1.__awaiter(void 0, void 0, void 0, function () {
            var dependencyFlag, _b, installing, failed, successful, installSpinner, workspacePackage, installPackageArgs, dependencies_1, dependencies_1_1, dependency, _c, workspaceRoot, installRootArgs, dependencies_2, dependencies_2_1, dependency, err_1;
            var e_1, _d, e_2, _e;
            return tslib_1.__generator(this, function (_f) {
                switch (_f.label) {
                    case 0:
                        dependencyFlag = dev ? '--dev' : '';
                        _b = new messages_1.InstallMessages(dependencies), installing = _b.installing, failed = _b.failed, successful = _b.successful;
                        installSpinner = ora_1.default(installing());
                        if (typeof dev === 'string') {
                            dependencies.push(dev);
                        }
                        installSpinner.start();
                        _f.label = 1;
                    case 1:
                        _f.trys.push([1, 4, , 5]);
                        return [4 /*yield*/, utils_1.findWorkspacePackageDir(true)];
                    case 2:
                        workspacePackage = _f.sent();
                        installPackageArgs = ['add', '--peer'];
                        try {
                            for (dependencies_1 = tslib_1.__values(dependencies), dependencies_1_1 = dependencies_1.next(); !dependencies_1_1.done; dependencies_1_1 = dependencies_1.next()) {
                                dependency = dependencies_1_1.value;
                                installPackageArgs.push(dependency);
                            }
                        }
                        catch (e_1_1) { e_1 = { error: e_1_1 }; }
                        finally {
                            try {
                                if (dependencies_1_1 && !dependencies_1_1.done && (_d = dependencies_1.return)) _d.call(dependencies_1);
                            }
                            finally { if (e_1) throw e_1.error; }
                        }
                        return [4 /*yield*/, execa_1.default('yarn', installPackageArgs, {
                                cwd: workspacePackage
                            })];
                    case 3:
                        _f.sent();
                        return [3 /*break*/, 5];
                    case 4:
                        _c = _f.sent();
                        return [3 /*break*/, 5];
                    case 5:
                        _f.trys.push([5, 8, , 9]);
                        return [4 /*yield*/, utils_1.findWorkspaceRootDir(true)];
                    case 6:
                        workspaceRoot = _f.sent();
                        installRootArgs = ['add', '--exact', '-W'];
                        if (dependencyFlag.length > 0) {
                            installRootArgs.push(dependencyFlag);
                        }
                        try {
                            for (dependencies_2 = tslib_1.__values(dependencies), dependencies_2_1 = dependencies_2.next(); !dependencies_2_1.done; dependencies_2_1 = dependencies_2.next()) {
                                dependency = dependencies_2_1.value;
                                installRootArgs.push(dependency);
                            }
                        }
                        catch (e_2_1) { e_2 = { error: e_2_1 }; }
                        finally {
                            try {
                                if (dependencies_2_1 && !dependencies_2_1.done && (_e = dependencies_2.return)) _e.call(dependencies_2);
                            }
                            finally { if (e_2) throw e_2.error; }
                        }
                        return [4 /*yield*/, execa_1.default('yarn', installRootArgs, {
                                cwd: workspaceRoot
                            })];
                    case 7:
                        _f.sent();
                        installSpinner.succeed(successful());
                        return [3 /*break*/, 9];
                    case 8:
                        err_1 = _f.sent();
                        installSpinner.fail(failed());
                        utils_1.logError(err_1);
                        process.exit(1);
                        return [3 /*break*/, 9];
                    case 9: return [2 /*return*/];
                }
            });
        });
    });
};
