"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var fs_extra_1 = tslib_1.__importDefault(require("fs-extra"));
var path_1 = tslib_1.__importDefault(require("path"));
var eslint_1 = require("eslint");
var utils_1 = require("../../shared/utils");
var lint_config_1 = require("../../configs/lint.config");
var messages_1 = require("../../shared/messages");
var package_const_1 = require("../../shared/constants/package.const");
exports.lintBinCommand = function (prog) {
    prog
        .command('lint')
        .describe('Lint a package. (default lint pattern src/**/*.{js,jsx,ts,tsx}')
        // eslint-disable-next-line @typescript-eslint/ban-ts-ignore
        // @ts-ignore
        .alias('l')
        .option('fix', 'Resolve fixable eslint errors')
        .example('lint --fix')
        .option('ignore-pattern', 'Ignore a pattern')
        .example('lint --ignore-pattern src/foo.ts')
        .action(function (opts) { return tslib_1.__awaiter(void 0, void 0, void 0, function () {
        var time, _a, linting, linted, files, rootDir, packageDir, e_1, packageJsonPath, eslintConfig, lintConfig, cli, report, duration;
        return tslib_1.__generator(this, function (_b) {
            switch (_b.label) {
                case 0:
                    time = process.hrtime();
                    _a = new messages_1.LintMessages(), linting = _a.linting, linted = _a.linted;
                    files = opts._.length > 0 ? opts._ : ['src/**/*.{js,jsx,ts,tsx}'];
                    return [4 /*yield*/, utils_1.findWorkspaceRootDir()];
                case 1:
                    rootDir = _b.sent();
                    _b.label = 2;
                case 2:
                    _b.trys.push([2, 4, , 5]);
                    return [4 /*yield*/, utils_1.findWorkspacePackageDir(true)];
                case 3:
                    packageDir = _b.sent();
                    return [3 /*break*/, 5];
                case 4:
                    e_1 = _b.sent();
                    packageDir = rootDir;
                    return [3 /*break*/, 5];
                case 5:
                    packageJsonPath = path_1.default.resolve(packageDir, package_const_1.PACKAGE_JSON);
                    return [4 /*yield*/, fs_extra_1.default.readJSON(packageJsonPath)];
                case 6:
                    eslintConfig = (_b.sent()).eslintConfig;
                    lintConfig = lint_config_1.createLintConfig({
                        dir: packageDir,
                        isRoot: rootDir === packageDir
                    });
                    cli = new eslint_1.CLIEngine({
                        baseConfig: tslib_1.__assign(tslib_1.__assign({}, lintConfig), (eslintConfig || {})),
                        extensions: ['.ts', '.tsx', '.js', '.jsx'],
                        fix: opts.fix,
                        ignorePattern: opts['ignore-pattern'],
                        parser: '@typescript-eslint/parser',
                        parserOptions: {
                            tsconfigRootDir: packageDir,
                            project: [
                                path_1.default.resolve(rootDir, package_const_1.TSCONFIG_JSON),
                                path_1.default.resolve(packageDir, package_const_1.TSCONFIG_JSON)
                            ]
                        }
                    });
                    console.log(linting(files));
                    report = cli.executeOnFiles(files);
                    if (opts.fix) {
                        eslint_1.CLIEngine.outputFixes(report);
                    }
                    console.log(cli.getFormatter()(report.results));
                    duration = process.hrtime(time);
                    console.log(linted(duration));
                    if (report.errorCount) {
                        process.exit(1);
                    }
                    return [2 /*return*/];
            }
        });
    }); });
};
