"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var execa_1 = tslib_1.__importDefault(require("execa"));
var os_1 = require("os");
var p_limit_1 = tslib_1.__importDefault(require("p-limit"));
var workspaces_messages_1 = require("../../shared/messages/workspaces.messages");
var utils_1 = require("../../shared/utils");
function workspacesBuildBinCommand(prog) {
    var _this = this;
    prog
        .command('workspaces build')
        .describe('Build each workspace')
        .example('workspaces build')
        // eslint-disable-next-line @typescript-eslint/ban-ts-ignore
        // @ts-ignore
        .alias('wb')
        .option('o, only', 'Build only specific workspaces')
        .option('e, exclude', 'Exclude specific workspaces')
        .option('q, quiet', 'Do not print any information about builds that are in the process', false)
        .example('workspaces build --self')
        .action(function (_a) {
        var quiet = _a.quiet;
        return tslib_1.__awaiter(_this, void 0, void 0, function () {
            var _b, introduce, compiled, compiling, failed, successful, entering, jobs, limit, packagesInfo, packagesLocationMap, packageJsons, _c, chunks, unprocessed, time, chunks_1, chunks_1_1, chunk, e_1_1, duration, error_1;
            var e_1, _d;
            var _this = this;
            return tslib_1.__generator(this, function (_e) {
                switch (_e.label) {
                    case 0:
                        _b = new workspaces_messages_1.WorkspacesMessages(), introduce = _b.introduce, compiled = _b.compiled, compiling = _b.compiling, failed = _b.failed, successful = _b.successful, entering = _b.entering;
                        jobs = Math.max(1, os_1.cpus().length / 2);
                        limit = p_limit_1.default(jobs);
                        return [4 /*yield*/, utils_1.getWorkspacesInfo()];
                    case 1:
                        packagesInfo = _e.sent();
                        packagesLocationMap = Object.fromEntries(packagesInfo.map(function (_a) {
                            var name = _a.name, location = _a.location;
                            return [name, location];
                        }));
                        return [4 /*yield*/, utils_1.readWorkspacePackages(packagesInfo)];
                    case 2:
                        packageJsons = _e.sent();
                        _c = utils_1.makeDependencyChunks(packageJsons), chunks = _c.chunks, unprocessed = _c.unprocessed;
                        utils_1.clearConsole();
                        _e.label = 3;
                    case 3:
                        _e.trys.push([3, 12, , 13]);
                        console.log(introduce());
                        console.log(compiling());
                        if (!quiet) {
                            utils_1.space();
                        }
                        time = process.hrtime();
                        _e.label = 4;
                    case 4:
                        _e.trys.push([4, 9, 10, 11]);
                        chunks_1 = tslib_1.__values(chunks), chunks_1_1 = chunks_1.next();
                        _e.label = 5;
                    case 5:
                        if (!!chunks_1_1.done) return [3 /*break*/, 8];
                        chunk = chunks_1_1.value;
                        return [4 /*yield*/, Promise.all(chunk.map(function (name) { return tslib_1.__awaiter(_this, void 0, void 0, function () {
                                var _this = this;
                                return tslib_1.__generator(this, function (_a) {
                                    return [2 /*return*/, limit(function () { return tslib_1.__awaiter(_this, void 0, void 0, function () {
                                            return tslib_1.__generator(this, function (_a) {
                                                switch (_a.label) {
                                                    case 0:
                                                        if (!quiet) {
                                                            console.log(entering(name));
                                                        }
                                                        return [4 /*yield*/, execa_1.default('yarn', ['build'], {
                                                                cwd: packagesLocationMap[name]
                                                            })];
                                                    case 1:
                                                        _a.sent();
                                                        if (!quiet) {
                                                            console.log(compiled(name));
                                                        }
                                                        return [2 /*return*/];
                                                }
                                            });
                                        }); })];
                                });
                            }); }))];
                    case 6:
                        _e.sent();
                        _e.label = 7;
                    case 7:
                        chunks_1_1 = chunks_1.next();
                        return [3 /*break*/, 5];
                    case 8: return [3 /*break*/, 11];
                    case 9:
                        e_1_1 = _e.sent();
                        e_1 = { error: e_1_1 };
                        return [3 /*break*/, 11];
                    case 10:
                        try {
                            if (chunks_1_1 && !chunks_1_1.done && (_d = chunks_1.return)) _d.call(chunks_1);
                        }
                        finally { if (e_1) throw e_1.error; }
                        return [7 /*endfinally*/];
                    case 11:
                        duration = process.hrtime(time);
                        if (!quiet) {
                            utils_1.space();
                        }
                        console.log(successful(duration));
                        utils_1.space();
                        return [3 /*break*/, 13];
                    case 12:
                        error_1 = _e.sent();
                        console.log(failed());
                        utils_1.logError(error_1);
                        return [3 /*break*/, 13];
                    case 13:
                        if (unprocessed.length > 0) {
                            console.log(utils_1.error("Potentially circular dependency\n      Please check the following packages attentively:\n      " + unprocessed.map(function (_a) {
                                var _b = tslib_1.__read(_a, 2), name = _b[0], dependencies = _b[1];
                                var _c;
                                return "   " + name + "  =>  " + ((_c = dependencies === null || dependencies === void 0 ? void 0 : dependencies.join(', ')) !== null && _c !== void 0 ? _c : '');
                            }).join("\n      ") + "\n      "));
                        }
                        return [2 /*return*/];
                }
            });
        });
    });
}
exports.workspacesBuildBinCommand = workspacesBuildBinCommand;
