"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var path_1 = tslib_1.__importDefault(require("path"));
var fs_extra_1 = tslib_1.__importDefault(require("fs-extra"));
var shelljs_1 = require("shelljs");
var models_1 = require("../models");
var error_utils_1 = require("./error.utils");
var package_const_1 = require("../constants/package.const");
exports.getWorkspacePackagePaths = function (workspaces) {
    var _a;
    if (workspaces === undefined) {
        return [];
    }
    if ('packages' in workspaces) {
        return (_a = workspaces.packages) !== null && _a !== void 0 ? _a : [];
    }
    return workspaces;
};
exports.getWorkspacePackageSetupPath = function (packages) {
    var rootPath = '/';
    var packageSetupPath = packages.reduce(function (path, pkg) {
        return pkg[pkg.length - 1] === '*' ? pkg.slice(0, pkg.length - 2) : path;
    }, rootPath);
    return packageSetupPath === rootPath ? 'packages' : packageSetupPath;
};
function findWorkspaceDir(possiblePath, conditionCallback) {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        var packageJsonPath, _a, _b;
        return tslib_1.__generator(this, function (_c) {
            switch (_c.label) {
                case 0:
                    // the path is too small
                    if (possiblePath.length < 10) {
                        return [2 /*return*/, null];
                    }
                    packageJsonPath = path_1.default.resolve(possiblePath, package_const_1.PACKAGE_JSON);
                    _a = fs_extra_1.default.existsSync(packageJsonPath);
                    if (!_a) return [3 /*break*/, 2];
                    _b = conditionCallback;
                    return [4 /*yield*/, fs_extra_1.default.readJSON(packageJsonPath)];
                case 1:
                    _a = _b.apply(void 0, [(_c.sent())]);
                    _c.label = 2;
                case 2: return [2 /*return*/, _a ? possiblePath
                        : findWorkspaceDir(path_1.default.resolve(possiblePath, '..'), conditionCallback)];
            }
        });
    });
}
exports.findWorkspaceRootDir = function (intercept) {
    if (intercept === void 0) { intercept = false; }
    return tslib_1.__awaiter(void 0, void 0, void 0, function () {
        var dir, _a, workspaceError;
        return tslib_1.__generator(this, function (_b) {
            switch (_b.label) {
                case 0:
                    _a = findWorkspaceDir;
                    return [4 /*yield*/, fs_extra_1.default.realpath(process.cwd())];
                case 1: return [4 /*yield*/, _a.apply(void 0, [_b.sent(),
                        function (pkg) { return pkg.workspaces !== undefined && pkg.private; }])];
                case 2:
                    dir = _b.sent();
                    if (dir === null) {
                        workspaceError = new models_1.NotFoundWorkspaceRootError();
                        if (intercept) {
                            throw workspaceError;
                        }
                        else {
                            error_utils_1.logError(workspaceError);
                            process.exit(1);
                        }
                    }
                    return [2 /*return*/, dir];
            }
        });
    });
};
exports.findWorkspacePackageDir = function (intercept) {
    if (intercept === void 0) { intercept = false; }
    return tslib_1.__awaiter(void 0, void 0, void 0, function () {
        var dir, _a, workspaceError;
        return tslib_1.__generator(this, function (_b) {
            switch (_b.label) {
                case 0:
                    _a = findWorkspaceDir;
                    return [4 /*yield*/, fs_extra_1.default.realpath(process.cwd())];
                case 1: return [4 /*yield*/, _a.apply(void 0, [_b.sent(),
                        function (pkg) { return pkg.workspace && !pkg.private; }])];
                case 2:
                    dir = _b.sent();
                    if (dir === null) {
                        workspaceError = new models_1.NotFoundPackageWorkspaceError();
                        if (intercept) {
                            throw workspaceError;
                        }
                        else {
                            error_utils_1.logError(workspaceError);
                            process.exit(1);
                        }
                    }
                    return [2 /*return*/, dir];
            }
        });
    });
};
function getWorkspacesInfo() {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        var yarnWorkspacesJsonInfo, yarnWorkspaces, rootDir;
        return tslib_1.__generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    yarnWorkspacesJsonInfo = shelljs_1.exec('yarn workspaces --json info', {
                        silent: true
                    }).stdout.trim();
                    yarnWorkspaces = JSON.parse(JSON.parse(yarnWorkspacesJsonInfo).data);
                    return [4 /*yield*/, exports.findWorkspaceRootDir()];
                case 1:
                    rootDir = _a.sent();
                    return [2 /*return*/, Object.keys(yarnWorkspaces).reduce(function (packages, name) { return tslib_1.__spread(packages, [
                            {
                                name: name,
                                location: path_1.default.resolve(rootDir, yarnWorkspaces[name].location)
                            }
                        ]); }, [])];
            }
        });
    });
}
exports.getWorkspacesInfo = getWorkspacesInfo;
